@@ -35,11 +35,11 @@ admin_ui <- function(id, lan = NULL) {
         ),
         tags$br(), tags$br(), tags$br(),
         DTOutput(outputId = ns("table_users")),
-        
+
         tags$br(),
 
         actionButton(
-          inputId = ns("remove_selected_allusers"),
+          inputId = ns("select_all_users"),
           # label = lan$get("Select all shown users"),
           label = "",
           class = "btn-secondary pull-right",
@@ -47,6 +47,14 @@ admin_ui <- function(id, lan = NULL) {
           icon = icon("check-square")
         ),
 
+        actionButton(
+          inputId = ns("edit_selected_users"),
+          label = lan$get("Edit selected users"),
+          class = "btn-primary pull-right disabled",
+          style = "margin-left: 5px",
+          icon = icon("pencil-square-o")
+        ),
+
         actionButton(
           inputId = ns("remove_selected_users"),
           label = lan$get("Remove selected users"),
@@ -80,14 +88,14 @@ admin_ui <- function(id, lan = NULL) {
         ),
 
         tags$br(),tags$br(), tags$br(), tags$hr(),
-        
+
         downloadButton(
           outputId = ns("download_sql_database"),
           label = lan$get("Download SQL database"),
           class = "btn-primary center-block",
           icon = icon("download")
         ),
-        
+
         tags$br(),tags$br()
 
       )
@@ -101,20 +109,20 @@ admin_ui <- function(id, lan = NULL) {
 #' @importFrom DBI dbConnect
 #' @importFrom RSQLite SQLite
 admin <- function(input, output, session, sqlite_path, passphrase, lan,
-                  inputs_list = NULL) {
+                  inputs_list = NULL, max_users = NULL) {
 
   ns <- session$ns
   jns <- function(x) {
     paste0("#", ns(x))
   }
-  
+
   token_start <- isolate(getToken(session = session))
 
   update_read_db <- reactiveValues(x = NULL)
 
   # read users table from database
   users <- reactiveVal(NULL)
-  
+
   observe({
     unbindDT(ns("table_users"))
     update_read_db$x
@@ -136,10 +144,10 @@ admin <- function(input, output, session, sqlite_path, passphrase, lan,
   observe({
     if(!is.null(users_update())) users(users_update())
   })
-  
+
   # read password management table from database
   pwds <- reactiveVal(NULL)
-  
+
   observe({
     unbindDT(ns("table_pwds"))
     update_read_db$x
@@ -170,15 +178,23 @@ admin <- function(input, output, session, sqlite_path, passphrase, lan,
     users <- users[, setdiff(names(users), c("password", "is_hashed_password")), drop = FALSE]
     users$Edit <- input_btns(ns("edit_user"), users$user, "Edit user", icon("pencil-square-o"), status = "primary", lan = lan())
     users$Remove <- input_btns(ns("remove_user"), users$user, "Delete user", icon("trash-o"), status = "danger", lan = lan())
-    users$Select <- input_checkbox_ui(ns("remove_mult_users"), users$user)
+    users$Select <- input_checkbox_ui(ns("select_mult_users"), users$user)
+    names_lan <- sapply(names(users), function(x) lan()$get(x))
+    change <- as.logical(users$admin)
+    users[change, "admin"] <- lan()$get("Yes")
+    users[!change, "admin"] <- lan()$get("No")
     datatable(
       data = users,
-      colnames = make_title(names(users)),
+      colnames = make_title(names_lan),
       rownames = FALSE,
       escape = FALSE,
       selection = "none",
+      style = "bootstrap",
       # extensions = 'FixedColumns', # bug using FixedColumns on checkbox + update table...
       options = list(
+        scrollY = if (nrow(users) > 10) "500px",
+        lengthChange = FALSE,
+        paging = FALSE,
         language = lan()$get_DT(),
         drawCallback = JS("function() {Shiny.bindAll(this.api().table().node());}"),
         # initComplete = JS(
@@ -195,13 +211,18 @@ admin <- function(input, output, session, sqlite_path, passphrase, lan,
   }, server = FALSE)
 
 
-  observeEvent(input$remove_selected_allusers, {
+  observeEvent(input$select_all_users, {
     input_names <- names(input)
-    remove_mult_users_input <- input_names[grep("^remove_mult_users", input_names)]
-    if(length(remove_mult_users_input) > 0){
-      ctrl <- lapply(remove_mult_users_input, function(x){
+    select_mult_users_names <- grep("^select_mult_users", input_names, value = TRUE)
+    select_mult_users_value <- unlist(lapply(select_mult_users_names, function(x) input[[x]]))
+    if (!all(select_mult_users_value)) {
+      ctrl <- lapply(select_mult_users_names, function(x) {
         updateCheckboxInput(session, x, value = TRUE)
       })
+    } else {
+      ctrl <- lapply(select_mult_users_names, function(x){
+        updateCheckboxInput(session, x, value = FALSE)
+      })
     }
   })
 
@@ -212,13 +233,24 @@ admin <- function(input, output, session, sqlite_path, passphrase, lan,
     pwds$`Change password` <- input_btns(ns("change_pwd"), pwds$user, "Ask to change password", icon("key"), status = "primary", lan = lan())
     pwds$`Reset password` <- input_btns(ns("reset_pwd"), pwds$user, "Reset password", icon("undo"), status = "warning", lan = lan())
     pwds$Select <- input_checkbox_ui(ns("change_mult_pwds"), pwds$user)
+    names_lan <- sapply(names(pwds), function(x) lan()$get(x))
+    change <- as.logical(pwds$must_change)
+    pwds[change, "must_change"] <- lan()$get("Yes")
+    pwds[!change, "must_change"] <- lan()$get("No")
+    change <- as.logical(pwds$have_changed)
+    pwds[change,"have_changed"] <- lan()$get("Yes")
+    pwds[!change,"have_changed"] <- lan()$get("No")
     datatable(
       data = pwds,
-      colnames = make_title(names(pwds)),
+      colnames = make_title(names_lan),
       rownames = FALSE,
       escape = FALSE,
       selection = "none",
+      style = "bootstrap",
       options = list(
+        scrollY = if (nrow(pwds) > 10) "500px",
+        lengthChange = FALSE,
+        paging = FALSE,
         language = lan()$get_DT(),
         drawCallback = JS("function() {Shiny.bindAll(this.api().table().node());}"),
         # initComplete = JS(
@@ -244,13 +276,15 @@ admin <- function(input, output, session, sqlite_path, passphrase, lan,
   })
 
   # Remove all selected users
-  r_selected_users <- callModule(module = input_checkbox, id = "remove_mult_users")
+  r_selected_users <- callModule(module = input_checkbox, id = "select_mult_users")
   observeEvent(r_selected_users(), {
     selected_users <- r_selected_users()
     if (length(selected_users) > 0) {
       toggleBtn(session = session, inputId = ns("remove_selected_users"), type = "enable")
+      toggleBtn(session = session, inputId = ns("edit_selected_users"), type = "enable")
     } else {
       toggleBtn(session = session, inputId = ns("remove_selected_users"), type = "disable")
+      toggleBtn(session = session, inputId = ns("edit_selected_users"), type = "disable")
     }
   })
 
@@ -300,11 +334,12 @@ admin <- function(input, output, session, sqlite_path, passphrase, lan,
 
 
 
-  # launch modal to edit informations about a user
+  # EDIT USER: launch modal to edit informations about a user ----
+
   observeEvent(input$edit_user, {
     users <- users()
     showModal(modalDialog(
-      title = "Edit user",
+      title = lan()$get("Edit user"),
       edit_user_ui(ns("edit_user"), credentials = users, username = input$edit_user, inputs_list = inputs_list, lan = lan()),
       tags$div(id = ns("placeholder-edituser-exist")),
       footer = tagList(
@@ -366,18 +401,19 @@ admin <- function(input, output, session, sqlite_path, passphrase, lan,
   })
 
 
-  # launch modal to add a new user
-  observeEvent(input$add_user, {
+
+
+  # EDIT MULTIPLE USERS: Launch modal to edit multiple users ----
+  observeEvent(input$edit_selected_users, {
     users <- users()
     showModal(modalDialog(
-      title = lan()$get("Add a user"),
-      edit_user_ui(ns("add_user"), users, NULL, inputs_list = inputs_list, lan = lan()),
-      tags$div(id = ns("placeholder-user-exist")),
+      title = "Edit user",
+      edit_user_ui(ns("edit_mult_user"), credentials = users, username = r_selected_users(), inputs_list = inputs_list, lan = lan()),
       footer = tagList(
         modalButton(lan()$get("Cancel")),
         actionButton(
-          inputId = ns("added_user"),
-          label = lan()$get("Confirm new user"),
+          inputId = ns("edited_mult_user"),
+          label = lan()$get("Confirm change"),
           class = "btn-primary",
           `data-dismiss` = "modal"
         )
@@ -385,6 +421,57 @@ admin <- function(input, output, session, sqlite_path, passphrase, lan,
     ))
   })
 
+  value_mult_edited <- callModule(module = edit_user, id = "edit_mult_user")
+
+  observeEvent(input$edited_mult_user, {
+    users <- users()
+    newval <- value_mult_edited$user
+    newval$user <- newval$admin <- NULL
+    conn <- dbConnect(SQLite(), dbname = sqlite_path)
+    on.exit(dbDisconnect(conn))
+    res_edit <- try({
+      for (user in r_selected_users()) {
+        users <- update_user(users, newval, user)
+      }
+      write_db_encrypt(conn = conn, value = users, name = "credentials", passphrase = passphrase)
+    }, silent = FALSE)
+    if (inherits(res_edit, "try-error")) {
+      showNotification(ui = lan()$get("Fail to update user"), type = "error")
+    } else {
+      showNotification(ui = lan()$get("User successfully updated"), type = "message")
+      update_read_db$x <- Sys.time()
+    }
+  })
+
+
+  # ADD NEW USER: launch modal to add a new user ----
+  observeEvent(input$add_user, {
+    users <- users()
+    if(!is.null(max_users) && is.numeric(max_users) && nrow(users) >= max_users){
+      showModal(
+        modalDialog(
+        title = lan()$get("Too many users"),
+        sprintf(lan()$get("Maximum number of users : %s"), max_users)
+        )
+      )
+    } else {
+      showModal(modalDialog(
+        title = lan()$get("Add a user"),
+        edit_user_ui(ns("add_user"), users, NULL, inputs_list = inputs_list, lan = lan()),
+        tags$div(id = ns("placeholder-user-exist")),
+        footer = tagList(
+          modalButton(lan()$get("Cancel")),
+          actionButton(
+            inputId = ns("added_user"),
+            label = lan()$get("Confirm new user"),
+            class = "btn-primary",
+            `data-dismiss` = "modal"
+          )
+        )
+      ))
+    }
+  })
+
   value_added <- callModule(module = edit_user, id = "add_user")
 
   # warning message if user already exist in database
@@ -433,7 +520,8 @@ admin <- function(input, output, session, sqlite_path, passphrase, lan,
       if(!"is_hashed_password" %in% colnames(users)){
         users$is_hashed_password <- FALSE
       }
-      newuser <- newuser[, colnames(users)]
+      newuser[setdiff(colnames(users), colnames(newuser))] <- NA
+      newuser <- newuser[colnames(users)]
       users <- rbind(users, newuser)
       write_db_encrypt(conn = conn, value = users, name = "credentials", passphrase = passphrase)
       resetpwd <- read_db_decrypt(conn = conn, name = "pwd_mngt", passphrase = passphrase)
