@@ -9,7 +9,7 @@
 #' @param tags_bottom A \code{tags (div, img, ...)} to be displayed on bottom of the authentication module.
 #' @param background A optionnal \code{css} for authentication background. See example.
 #' @param choose_language \code{logical/character}. Add language selection on top ? TRUE for all supported languages
-#' or a vector of possibilities like \code{c("fr", "en")}. If enabled, \code{input$shinymanager_language} is created
+#' or a vector of possibilities like \code{c("en", "fr", "pt-BR", "es", "de", "pl")}. If enabled, \code{input$shinymanager_language} is created
 #' @param ... : Used for old version compatibility.
 #' 
 #' 
@@ -61,38 +61,47 @@ auth_ui <- function(id, status = "primary", tags_top = NULL,
             tags$div(
               class = "panel-body",
               {
+                
                 choices = lan$get_language()
+                lan_registered <- lan$get_language_registered()
                 if(is.logical(choose_language) && choose_language){
-                  choices = lan$get_language_registered()
+                  choices = unname(lan$get_language_registered())
                 } else if(is.character(choose_language)){
-                  choices = unique(c(intersect(choose_language, lan$get_language_registered()), lan$get_language()))
+                  choices = unique(c(intersect(choose_language, unname(lan$get_language_registered())), lan$get_language()))
                 }
                 
-                  selected = ifelse(lan$get_language() %in% choices, 
-                                    lan$get_language(),
-                                    choices[1])
-                  if(length(choices) == 1){
-                    style = "display:none"
-                  } else {
-                    style = "margin-bottom:-50px;"
+                names(choices) <- choices
+                for(i in 1:length(choices)){
+                  ind <- which(lan_registered %in% choices[i])
+                  if(length(ind) > 0){
+                    names(choices)[i] <- names(lan_registered)[ind]
                   }
-                  tags$div(style = style,
-                           fluidRow(
-                             column(width = 3, offset = 6, uiOutput(ns("label_language"))),
-                             column(3,
-                                    tags$div(
-                                      style = "text-align: left; font-size: 12px;",
-                                      selectInput(
-                                        inputId = ns("language"),
-                                        label = NULL,
-                                        choices = choices,
-                                        selected = selected,
-                                        width = "100%"
-                                      )
+                }
+                selected = ifelse(lan$get_language() %in% choices, 
+                                  lan$get_language(),
+                                  choices[1])
+                if(length(choices) == 1){
+                  style = "display:none"
+                } else {
+                  style = "margin-bottom:-50px;"
+                }
+                tags$div(style = style,
+                         fluidRow(
+                           column(width = 4, offset = 4, uiOutput(ns("label_language"))),
+                           column(4,
+                                  tags$div(
+                                    style = "text-align: left; font-size: 12px;",
+                                    selectInput(
+                                      inputId = ns("language"),
+                                      label = NULL,
+                                      choices = choices,
+                                      selected = selected,
+                                      width = "100%"
                                     )
-                             )
+                                  )
                            )
-                  )
+                         )
+                )
               },
               tags$div(
                 style = "text-align: center;",
@@ -137,10 +146,16 @@ auth_ui <- function(id, status = "primary", tags_top = NULL,
 #' @param input,output,session Standard Shiny server arguments.
 #' @param check_credentials Function with two arguments (\code{user},
 #'  the username provided by the user and \code{password}, his/her password).
-#'  Must return \code{TRUE} or \code{FALSE}.
-#'  To use additionnals arguments, set them with \code{purrr::partial} (see examples).
+#'  Must return a \code{list} with at least 4 slots :
+#'  \itemize{
+#'   \item \strong{result} : logical, result of authentication.
+#'   \item \strong{expired} : logical, is user has expired ? Always \code{FALSE} if \code{db} doesn't have a \code{expire} column.
+#'   \item \strong{authorized} : logical, is user can access to his app ? Always \code{TRUE} if \code{db} doesn't have a \code{applications} column.
+#'   \item \strong{user_info} : the line in \code{db} corresponding to the user.
+#'  }
+#'  
 #' @param use_token Add a token in the URL to check authentication. Should not be used directly.
-#' @param lan An langauge object. Should not be used directly.
+#' @param lan A language object. See  \code{\link{use_language}}
 #' 
 #' @export
 #'
@@ -156,7 +171,8 @@ auth_ui <- function(id, status = "primary", tags_top = NULL,
 #' @importFrom htmltools tags
 #' @importFrom shiny reactiveValues observeEvent removeUI updateQueryString insertUI is.reactive icon updateActionButton updateTextInput renderUI
 #' @importFrom stats setNames
-auth_server <- function(input, output, session, check_credentials, 
+auth_server <- function(input, output, session, 
+                        check_credentials, 
                         use_token = FALSE, lan = NULL) {
   
   ns <- session$ns
@@ -164,11 +180,12 @@ auth_server <- function(input, output, session, check_credentials,
     paste0("#", ns(x))
   }
   
+  
   if(!is.reactive(lan)){
     if(is.null(lan)){
-      lan <- reactive(use_language())
+      lan <- reactiveVal(use_language())
     } else {
-      lan <- reactive(lan)
+      lan <- reactiveVal(lan)
     }
   }
   
@@ -224,7 +241,7 @@ auth_server <- function(input, output, session, check_credentials,
       if (isTRUE(use_token)) {
         # add_token(token, as.list(res_auth$user_info))
         .tok$add(token, as.list(res_auth$user_info))
-        updateQueryString(queryString = paste0("?token=", token, "&language=", lan()$get_language()), session = session)
+        addAuthToQuery(session, token, lan()$get_language())
         session$reload()
       }
       
