@@ -10,7 +10,8 @@
 #' @param theme Alternative Bootstrap stylesheet, default is to use \code{readable},
 #'  you can use themes provided by \code{shinythemes}.
 #'  It will affect the authentication panel and the admin page.
-#' @param language Language to use for labels, supported values are : "en", "fr", "pt-BR".
+#' @param language Language to use for labels, supported values are : "en", "fr", "pt-BR", "es", "de", "pl".
+#' @param fab_position Position for the FAB button, see \code{\link{fab_button}} for options.
 #'
 #' @note A special input value will be accessible server-side with \code{input$shinymanager_where}
 #'  to know in which step user is : authentication, application, admin or password.
@@ -25,12 +26,18 @@
 #' @name secure-app
 #'
 #' @example examples/secure_app.R
-secure_app <- function(ui, ..., enable_admin = FALSE, head_auth = NULL, theme = NULL, language = "en") {
-  if (!language %in% c("en", "fr", "pt-BR")) {
-    warning("Only supported language for the now are: en, fr, pt-BR", call. = FALSE)
+secure_app <- function(ui,
+                       ...,
+                       enable_admin = FALSE,
+                       head_auth = NULL,
+                       theme = NULL,
+                       language = "en",
+                       fab_position = "bottom-right") {
+  if (!language %in% c("en", "fr", "pt-BR", "es", "de", "pl")) {
+    warning("Only supported language for the now are: en, fr, pt-BR, es, de, pl", call. = FALSE)
     language <- "en"
   }
-  
+
   lan <- use_language(language)
   ui <- force(ui)
   enable_admin <- force(enable_admin)
@@ -38,14 +45,14 @@ secure_app <- function(ui, ..., enable_admin = FALSE, head_auth = NULL, theme =
   if (is.null(theme)) {
     theme <- "shinymanager/css/readable.min.css"
   }
-  
+
   function(request) {
     query <- parseQueryString(request$QUERY_STRING)
-    token <- query$token
+    token <- gsub('\"', "", query$token)
     admin <- query$admin
     language <- query$language
     if(!is.null(language)){
-      lan <- use_language(language)
+      lan <- use_language(gsub('\"', "", language))
     }
     if (.tok$is_valid(token)) {
       is_forced_chg_pwd <- is_force_chg_pwd(token = token)
@@ -69,19 +76,18 @@ secure_app <- function(ui, ..., enable_admin = FALSE, head_auth = NULL, theme =
           header = tagList(
             tags$style(".navbar-header {margin-left: 16.66% !important;}"),
             fab_button(
+              position = fab_position,
               actionButton(
                 inputId = ".shinymanager_logout",
-                label = NULL,
-                tooltip = lan$get("Logout"),
+                label = lan$get("Logout"),
                 icon = icon("sign-out")
               ),
               actionButton(
                 inputId = ".shinymanager_app",
-                label = NULL,
-                tooltip = lan$get("Go to application"),
+                label = lan$get("Go to application"),
                 icon = icon("share")
               )
-            ), 
+            ),
             shinymanager_where("admin")
           ),
           tabPanel(
@@ -91,7 +97,7 @@ secure_app <- function(ui, ..., enable_admin = FALSE, head_auth = NULL, theme =
             shinymanager_language(lan$get_language())
           ),
           tabPanel(
-            title = "Logs",
+            title = lan$get("Logs"),
             logs_ui("logs", lan),
             shinymanager_language(lan$get_language())
           )
@@ -99,16 +105,15 @@ secure_app <- function(ui, ..., enable_admin = FALSE, head_auth = NULL, theme =
       } else {
         if (isTRUE(enable_admin) && .tok$is_admin(token) && !is.null(.tok$get_sqlite_path())) {
           menu <- fab_button(
+            position = fab_position,
             actionButton(
               inputId = ".shinymanager_logout",
-              label = NULL,
-              tooltip = lan$get("Logout"),
+              label = lan$get("Logout"),
               icon = icon("sign-out")
             ),
             actionButton(
               inputId = ".shinymanager_admin",
-              label = NULL,
-              tooltip = lan$get("Administrator mode"),
+              label = lan$get("Administrator mode"),
               icon = icon("cogs")
             )
           )
@@ -117,10 +122,10 @@ secure_app <- function(ui, ..., enable_admin = FALSE, head_auth = NULL, theme =
             warning("Admin mode is only available when using a SQLite database!", call. = FALSE)
           }
           menu <- fab_button(
+            position = fab_position,
             actionButton(
               inputId = ".shinymanager_logout",
-              label = NULL,
-              tooltip = lan$get("Logout"),
+              label = lan$get("Logout"),
               icon = icon("sign-out")
             )
           )
@@ -164,41 +169,60 @@ secure_app <- function(ui, ..., enable_admin = FALSE, head_auth = NULL, theme =
 #' @param check_credentials Function passed to \code{\link{auth_server}}.
 #' @param timeout Timeout session (minutes) before logout if sleeping. Defaut to 15. 0 to disable.
 #' @param inputs_list \code{list}. If database credentials, you can configure inputs for editing users information. See Details.
+#' @param max_users \code{integer}. If not NULL, maximum of users in sql credentials.
 #' @param fileEncoding 	character string: Encoding of logs downloaded file. See \code{\link{write.table}}
+#' @param keep_token Logical, keep the token used to authenticate in the URL, it allow to refresh the
+#'  application in the browser, but careful the token can be shared between users ! Default to \code{FALSE}.
 #' @param session Shiny session.
-#' 
-#' @details 
-#' 
-#' If database credentials, you can configure inputs with \code{inputs_list} for editing users information 
-#' from the admin console. \code{start}, \code{expire}, \code{admin} and \code{password} are not configurable. 
+#'
+#' @details
+#'
+#' If database credentials, you can configure inputs with \code{inputs_list} for editing users information
+#' from the admin console. \code{start}, \code{expire}, \code{admin} and \code{password} are not configurable.
 #' The others columns are rendering by defaut using a \code{textInput}. You can modify this using \code{inputs_list}.
 #' \code{inputs_list} must be a named list. Each name must be a column name, and then we must have the function
-#'  shiny to call \code{fun} and the arguments \code{args} like this : 
+#'  shiny to call \code{fun} and the arguments \code{args} like this :
 #'  \code{
 #'  list(group = list(
-#'      fun = "selectInput", 
+#'      fun = "selectInput",
 #'      args = list(
-#'          choices = c("all", "restricted"), 
-#'          multiple = TRUE, 
+#'          choices = c("all", "restricted"),
+#'          multiple = TRUE,
 #'          selected = c("all", "restricted")
 #'       )
 #'      )
 #' )
 #' }
-#' 
+#'
 #' @export
 #'
 #' @importFrom shiny callModule getQueryString parseQueryString
 #'  updateQueryString observe getDefaultReactiveDomain isolate invalidateLater
 #'
 #' @rdname secure-app
-secure_server <- function(check_credentials, timeout = 15, inputs_list = NULL,
+secure_server <- function(check_credentials,
+                          timeout = 15,
+                          inputs_list = NULL,
+                          max_users = NULL,
                           fileEncoding = "",
+                          keep_token = FALSE,
                           session = shiny::getDefaultReactiveDomain()) {
+
+  session$setBookmarkExclude(c(session$getBookmarkExclude(),
+                               "shinymanager_language", 
+                               ".shinymanager_timeout", 
+                               ".shinymanager_admin",
+                               ".shinymanager_logout", 
+                               "shinymanager_where"))
   
-  isolate(resetQueryString(session = session))
   token_start <- isolate(getToken(session = session))
-  
+  if (isTRUE(keep_token)) {
+    .tok$reset_count(token_start)
+  } else {
+    isolate(resetQueryString(session = session))
+  }
+
+
   lan <- reactiveVal(use_language())
   observe({
     lang <- getLanguage(session = session)
@@ -206,48 +230,49 @@ secure_server <- function(check_credentials, timeout = 15, inputs_list = NULL,
       lan(use_language(lang))
     }
   })
-  
+
   callModule(
     module = auth_server,
     id = "auth",
     check_credentials = check_credentials,
-    use_token = TRUE, 
+    use_token = TRUE,
     lan = lan
   )
-  
+
   callModule(
     module = pwd_server,
     id = "password",
     user = reactiveValues(user = .tok$get(token_start)$user),
     update_pwd = update_pwd,
-    use_token = TRUE, 
+    use_token = TRUE,
     lan = lan
   )
-  
+
   .tok$set_timeout(timeout)
-  
+
   path_sqlite <- .tok$get_sqlite_path()
   if (!is.null(path_sqlite)) {
     callModule(
       module = admin,
       id = "admin",
       sqlite_path = path_sqlite,
-      passphrase = .tok$get_passphrase(), 
-      inputs_list = inputs_list, 
+      passphrase = .tok$get_passphrase(),
+      inputs_list = inputs_list,
+      max_users = max_users,
       lan = lan
     )
     callModule(
       module = logs,
       id = "logs",
       sqlite_path = path_sqlite,
-      passphrase = .tok$get_passphrase(), 
-      fileEncoding = fileEncoding, 
+      passphrase = .tok$get_passphrase(),
+      fileEncoding = fileEncoding,
       lan = lan
     )
   }
-  
+
   user_info_rv <- reactiveValues()
-  
+
   observe({
     token <- getToken(session = session)
     if (!is.null(token)) {
@@ -267,21 +292,21 @@ secure_server <- function(check_credentials, timeout = 15, inputs_list = NULL,
       }
     }
   })
-  
+
   observeEvent(session$input$.shinymanager_admin, {
     token <- getToken(session = session)
-    updateQueryString(queryString = sprintf("?token=%s&admin=true&language=%s", token, lan()$get_language()), session = session, mode = "replace")
+    updateQueryString(queryString = sprintf("?token=\"%s\"&admin=true&language=\"%s\"", token, lan()$get_language()), session = session, mode = "replace")
     .tok$reset_count(token)
     session$reload()
   }, ignoreInit = TRUE)
-  
+
   observeEvent(session$input$.shinymanager_app, {
     token <- getToken(session = session)
-    updateQueryString(queryString = sprintf("?token=%s&language=%s", token, lan()$get_language()), session = session, mode = "replace")
+    updateQueryString(queryString = sprintf("?token=\"%s\"&language=\"%s\"", token, lan()$get_language()), session = session, mode = "replace")
     .tok$reset_count(token)
     session$reload()
   }, ignoreInit = TRUE)
-  
+
   observeEvent(session$input$.shinymanager_logout, {
     token <- getToken(session = session)
     logout_logs(token)
@@ -289,11 +314,11 @@ secure_server <- function(check_credentials, timeout = 15, inputs_list = NULL,
     clearQueryString(session = session)
     session$reload()
   }, ignoreInit = TRUE)
-  
-  
-  
+
+
+
   if (timeout > 0) {
-    
+
     observeEvent(session$input$.shinymanager_timeout, {
       token <- getToken(session = session)
       if (!is.null(token)) {
@@ -305,7 +330,7 @@ secure_server <- function(check_credentials, timeout = 15, inputs_list = NULL,
         }
       }
     })
-    
+
     observe({
       invalidateLater(30000, session)
       token <- getToken(session = session)
@@ -318,9 +343,9 @@ secure_server <- function(check_credentials, timeout = 15, inputs_list = NULL,
         }
       }
     })
-    
+
   }
-  
+
   return(user_info_rv)
 }
 
