@@ -36,6 +36,10 @@ test_that("edit_user UI works", {
 
   edit_user_ui_tags <- edit_user_ui(id = "edit", credentials = credentials)
 
+  edit_user_ui_tags <- edit_user_ui(id = "edit", 
+                                    credentials = credentials, 
+                                    username = "fanny")
+  
   expect_is(edit_user_ui_tags, "shiny.tag.list")
 })
 
